@@ -1,17 +1,27 @@
-"Test for template tags."
-from __future__ import unicode_literals
+# -*- coding: utf-8 -*-
 
+"Test for template tags."
 from django.contrib.auth.models import Permission
 from django.core.cache import cache
 from django.template import Template, TemplateSyntaxError
 from django.template.context import RequestContext
 from django.test.client import RequestFactory
-from django.utils.unittest import skipIf
+from unittest import skipIf
 
 from . import DaysLog
 from .base import ScribblerDataTestCase
 from scribbler.conf import CACHE_TIMEOUT
 
+class UnicodeURLTestCase(ScribblerDataTestCase):
+    "Test, that unicode characters in url got cached"
+
+    def testUnicodeURL(self):
+        scribble = self.create_scribble(
+            url='/foo/čřžžýü', slug='sidebar',
+            content='<p>Scribble content.</p>'
+        )
+        self.assertEquals(scribble.url, "/foo/čřžžýü")
+
 
 class RenderScribbleTestCase(ScribblerDataTestCase):
     "Tag to render a scribble for the current page."
@@ -70,6 +80,16 @@ class RenderScribbleTestCase(ScribblerDataTestCase):
         result = self.render_template_tag(slug='"sidebar"')
         self.assertTrue('<p>Default.</p>' in result)
 
+    def test_unicode_rendering(self):
+        "Render with unicode defaults when no scribbles exist."
+        # On Django>=1.9 ScribbleFormMixin.clean_content directly uses django.template.Template
+        # and also uses force_text that may fail for non-string objects that have __str__ with
+        # unicode output.
+        self.scribble.delete()
+        unicode_default = '<p>\u0422\u0435\u043a\u0441\u0442.</p>'
+        result = self.render_template_tag(slug='"sidebar"', default=unicode_default)
+        self.assertTrue(unicode_default in result)
+
     def test_no_slug_given(self):
         "Slug is required by the tag."
         self.assertRaises(TemplateSyntaxError, self.render_template_tag, slug='')
