@@ -1,22 +1,32 @@
 "Tests for preview/save views."
-from __future__ import unicode_literals
-
+import os
 import json
 from datetime import date
+import time
+from unittest import skipIf
 
 from django.contrib.auth.models import Permission, User
 from django.contrib.contenttypes.models import ContentType
-from django.core.urlresolvers import reverse
+from django.urls import reverse
+from django.contrib.staticfiles.testing import StaticLiveServerTestCase
+from django.test import override_settings
+
+from selenium import webdriver
+from selenium.webdriver.common.by import By
+from selenium.webdriver.common.keys import Keys
+from selenium.webdriver.common.action_chains import ActionChains
+from selenium.webdriver.support.ui import WebDriverWait
+from selenium.webdriver.support import expected_conditions as EC
+
 
 from . import DaysLog
 from .base import ScribblerDataTestCase, Scribble
 
 
+@override_settings(ROOT_URLCONF='scribbler.tests.urls')
 class BaseViewTestCase(ScribblerDataTestCase):
     "Common functionality for testing views."
 
-    urls = "scribbler.tests.urls"
-
     def setUp(self):
         self.user = self.create_user(username='test', password='test')
         self.client.login(username='test', password='test')
@@ -26,7 +36,7 @@ class BaseViewTestCase(ScribblerDataTestCase):
             content_type__model='scribble',
         )
         self.add_perm = Permission.objects.get(
-            codename='add_scribble', 
+            codename='add_scribble',
             content_type__app_label='scribbler',
             content_type__model='scribble',
         )
@@ -51,7 +61,8 @@ class PreviewTestCase(BaseViewTestCase):
 
     def setUp(self):
         super(PreviewTestCase, self).setUp()
-        self.url = reverse('preview-scribble')
+        content_type = ContentType.objects.get_for_model(Scribble)
+        self.url = reverse('preview-scribble', args=(content_type.pk,))
 
     def get_valid_data(self):
         "Base valid data."
@@ -96,13 +107,25 @@ class PreviewTestCase(BaseViewTestCase):
         self.assertEqual(response.status_code, 403)
 
     def test_permission_required(self):
-        "Return 403 if user is does not have permissions to preview scribbles."
+        "Return 403 if user does not have permissions to preview scribbles."
         self.user.user_permissions.remove(self.change_perm)
         self.user.user_permissions.remove(self.add_perm)
         data = self.get_valid_data()
         response = self.client.post(self.url, data=data)
         self.assertEqual(response.status_code, 403)
 
+    def test_permission_required_scribble_field(self):
+        "Return 403 if user does not have permissions to preview a scribble_field."
+        days_log = DaysLog.objects.create(happenings=self.get_random_string())
+        content_type = ContentType.objects.get_for_model(days_log)
+        url = reverse('preview-scribble', args=(content_type.pk,))
+
+        self.user.user_permissions.remove(self.change_dayslog_perm)
+        data = {'content': self.get_random_string()}
+
+        response = self.client.post(url, data=data)
+        self.assertEqual(response.status_code, 403)
+
     def test_preview_existing(self):
         "Preview content for a scribble which exists. See #34."
         data = self.get_valid_data()
@@ -392,3 +415,59 @@ class DeleteTestCase(BaseViewTestCase):
         data = self.get_valid_data()
         response = self.client.post(self.url, data=data)
         self.assertEqual(response.status_code, 403)
+
+
+@skipIf(True, 'PhantomJS was deprecated')
+@override_settings(ROOT_URLCONF='scribbler.tests.urls')
+class FunctionalTestCase(StaticLiveServerTestCase, BaseViewTestCase):
+
+    def setUp(self):
+        super(FunctionalTestCase, self).setUp()
+        self.browser = webdriver.PhantomJS()
+        self.browser.implicitly_wait(3)
+
+    def tearDown(self):
+        self.browser.quit()
+
+    @skipIf(os.environ.get('TRAVIS'), 'selenium is too flaky for this to work over the full matrix in CI')
+    def test_editor(self):
+        self.browser.get('%s%s' % (self.live_server_url, '/test/'))
+        username_input = self.browser.find_element_by_name("username")
+        username_input.send_keys('test')
+        password_input = self.browser.find_element_by_name("password")
+        password_input.send_keys('test')
+        self.browser.find_element_by_name('submit').click()
+        self.browser.implicitly_wait(10)
+        scribble = self.browser.find_element_by_class_name("scribble-wrapper")
+        self.assertTrue(scribble)
+        self.browser.implicitly_wait(10)
+        editor = self.browser.find_element_by_id("scribbleEditorContainer")
+        self.assertTrue(editor)
+        scribble.click()
+        time.sleep(1)
+        self.assertIn("height: 300px", editor.get_attribute('style'))
+        self.browser.implicitly_wait(10)
+        action = ActionChains(self.browser)
+        action.send_keys(Keys.ARROW_DOWN)
+        action.send_keys(Keys.ARROW_DOWN)
+        action.send_keys("<")
+        action.send_keys("p")
+        action.send_keys(">")
+        action.send_keys("This is a Test")
+        action.send_keys("<")
+        action.send_keys("/")
+        action.send_keys("p")
+        action.send_keys(">")
+        action.perform()
+        wait = WebDriverWait(self.browser, 10)
+        text = wait.until(EC.text_to_be_present_in_element((By.CSS_SELECTOR,
+                                                            'div.scribble-content p:nth-child(2)'), "This is a Test"))
+        self.assertTrue(text)
+        self.browser.find_element_by_class_name("save").click()
+        scribble.click()
+        time.sleep(1)
+        action = ActionChains(self.browser)
+        action.send_keys(Keys.F11)
+        action.perform()
+        self.browser.implicitly_wait(10)
+        self.assertTrue(self.browser.find_element_by_class_name("CodeMirror-fullscreen"))
