@@ -1,11 +1,10 @@
-from __future__ import unicode_literals
-
 import json
 
+from django import template
 from django.core.serializers.json import DjangoJSONEncoder
 from django.http import HttpResponse, HttpResponseForbidden
 from django.shortcuts import get_object_or_404
-from django.template import RequestContext, Template
+from django.template import RequestContext
 from django.views.debug import ExceptionReporter
 from django.views.decorators.http import require_POST
 from django.contrib.contenttypes.models import ContentType
@@ -15,21 +14,27 @@ from .models import Scribble
 from .utils import get_variables
 
 
-def build_scribble_context(scribble, request):
+def build_scribble_context(scribble):
     "Create context for rendering a scribble or scribble preview."
     context = {
         'scribble': scribble,
     }
-    return RequestContext(request, context)
+
+    return context
 
 
 @require_POST
-def preview_scribble(request):
+def preview_scribble(request, ct_pk):
     "Render scribble content or return error information."
-    if not request.user.is_authenticated():
+    if not request.user.is_authenticated:
         return HttpResponseForbidden()
-    can_edit = request.user.has_perm('scribbler.change_scribble')
-    can_create = request.user.has_perm('scribbler.add_scribble')
+    content_type = get_object_or_404(ContentType, pk=ct_pk)
+    change_scribble = '{0}.change_{1}'.format(
+        content_type.app_label, content_type.model)
+    add_scribble = '{0}.add_{1}'.format(
+        content_type.app_label, content_type.model)
+    can_edit = request.user.has_perm(change_scribble)
+    can_create = request.user.has_perm(add_scribble)
     if not (can_edit or can_create):
         return HttpResponseForbidden()
     results = {
@@ -39,16 +44,29 @@ def preview_scribble(request):
     form = PreviewForm(request.POST)
     if form.is_valid():
         results['valid'] = True
-        template = Template(form.cleaned_data.get('content', ''))
-        context = build_scribble_context(form.instance, request)
-        results['html'] = template.render(context)
-        results['variables'] = get_variables(context)
+        if hasattr(template, 'engines'):
+            scribbler_template = template.engines['django'].from_string(form.cleaned_data.get('content', ''))
+        else:
+            scribbler_template = template.Template(form.cleaned_data.get('content', ''))
+        context = build_scribble_context(form.instance)
+        results['html'] = scribbler_template.render(context, request)
+        results['variables'] = get_variables(RequestContext(request, context))
     else:
         if hasattr(form, 'exc_info'):
-            exc_type, exc_value, tb = form.exc_info
-            reporter = ExceptionReporter(request, exc_type, exc_value, tb)
-            reporter.get_template_exception_info()
-            results['error'] = reporter.template_info
+            # Pre Django 1.9
+            try:
+                exc_type, exc_value, tb = form.exc_info
+                reporter = ExceptionReporter(request, exc_type, exc_value, tb)
+                reporter.get_template_exception_info()
+                results['error'] = reporter.template_info
+            # Django >= 1.9: get_template_info() is moved from ExceptionReporter
+            # onto Template. We pass the data it returns from scribbler/forms.py
+            # to here.
+            except (ValueError, AttributeError):
+                # ValueError is raised when we pass in all 12 the arguments,
+                # in form.exc_info and AttributeError is raised when
+                # ExceptionReporter.get_template_exception_info() is called.
+                results['error'] = form.exc_info
         else:
             # Not sure what to do here
             results['error'] = {
@@ -62,7 +80,7 @@ def preview_scribble(request):
 @require_POST
 def create_edit_scribble(request, scribble_id=None):
     "Create a new Scribble or edit an existing one."
-    if not request.user.is_authenticated():
+    if not request.user.is_authenticated:
         return HttpResponseForbidden()
     if scribble_id is not None:
         scribble = get_object_or_404(Scribble, pk=scribble_id)
@@ -86,7 +104,7 @@ def create_edit_scribble(request, scribble_id=None):
 
 @require_POST
 def edit_scribble_field(request, ct_pk, instance_pk, field_name):
-    if not request.user.is_authenticated():
+    if not request.user.is_authenticated:
         return HttpResponseForbidden()
     content_type = get_object_or_404(ContentType, pk=ct_pk)
     perm_name = '{0}.change_{1}'.format(content_type.app_label, content_type.model)
@@ -112,7 +130,7 @@ def edit_scribble_field(request, ct_pk, instance_pk, field_name):
 @require_POST
 def delete_scribble(request, scribble_id):
     "Delete an existing scribble."
-    if not request.user.is_authenticated():
+    if not request.user.is_authenticated:
         return HttpResponseForbidden()
     scribble = get_object_or_404(Scribble, pk=scribble_id)
     if not request.user.has_perm('scribbler.delete_scribble'):
